from django.contrib.auth import models as auth_models
from django.utils.cache import patch_vary_headers
from django.template.loader import render_to_string
from pagehelp import models, utils
from django.contrib.sites.models import Site
from django.conf import settings
from django.utils.encoding import smart_str
import re

KEY = '_pagehelp_anonymous'
RE_APPEND = re.compile('</body>', re.IGNORECASE)


class PageMiddleware(object):
    """
    Middleware which adds the contextual help for this page to the response. 

    Additionally, when an anonymous user is logged in, any page display
    settings in the cookie are linked with the user.

    """

    def process_request(self, request):
        if not request.user.is_authenticated():
            setattr(request, KEY, True)

    def process_response(self, request, response):
        user = hasattr(request, 'session') and request.user
        if getattr(request, KEY, None):
            if isinstance(user, auth_models.User):
                default, pages = utils.decode_cookie(request)
                # Set the default display.
                utils.set_user_display(request, default, models.UserDefault)
                # Set the display for each page.
                for page_id, display in pages.items():
                    try:
                        page = models.Page.on_site.get(pk=page_id)
                    except models.Page.DoesNotExist:
                        continue
                    utils.set_user_display(request, display, models.UserPage,
                                           page=page)
                utils.encode_cookie(request, response, default, pages)
        if user and user.is_authenticated():
            site_setting = models.SiteSetting.objects.get_current()
            can_edit = site_setting.can_edit(user)
            can_delete = site_setting.can_delete(user)
        else:
            can_edit = can_delete = False
        page = utils.get_page(request.path)
        if page or can_edit:
            static_url = getattr(settings, 'STATIC_URL', settings.MEDIA_URL)
            context = {'page': page, 'STATIC_URL': static_url,
                       'can_edit': can_edit, 'can_delete': can_delete,
                       'default_open': utils.get_default_display(request)}
            # TODO: add has_jquery to the context
            if page:
                context['next'] = request.get_full_path()
                context['open'] = utils.is_visible(request, page)
            else:
                context['path'] = request.path
                context['site_id'] = Site.objects.get_current().pk
            code = render_to_string('pagehelp.html', context)
            response.content, n = RE_APPEND.subn(smart_str(r'%s\0' % code),
                response.content, count=1)
            if n:
                # Output may vary depending on the cookie, so set the Vary
                # header.
                patch_vary_headers(response, ('Cookie',))
    
                # Since the content has been modified, any Etag will now be
                # incorrect.  We could recalculate, but only if we assume that
                # the Etag was set by CommonMiddleware. The safest thing is
                # just to delete. See bug #9163
                del response['ETag']
        return response



from django import http
from django.contrib.auth.models import User
from django.contrib.sites.models import Site
from pagehelp import models, utils


def _next_page(request):
    """
    Return the context 

    """
    next = request.REQUEST.get('next')
    if next and not '//' in next:
        return next


def set_default(request, show):
    """
    Set a user's default display setting. 

    """
    next = _next_page(request)
    if not next or request.method != 'POST':
        raise http.Http404

    set_cookie = False
    if isinstance(request.user, User):
        utils.set_user_display(request, show, models.UserDefault,
                               site=Site.objects.get_current())
        page = utils.get_page(next)
        if page:
            if isinstance(request.user, User):
                utils.set_user_display(request, show, models.UserPage,
                                       page=page)
    else:
        set_cookie = True
        default, pages = utils.decode_cookie(request)
        default = show
    response = http.HttpResponseRedirect(next)
    if set_cookie:
        utils.encode_cookie(request, response, default, pages)
    return response


def set_page(request, show):
    """
    Set a user's display setting for an individual page. 

    """
    next = _next_page(request)
    if not next or request.method != 'POST':
        raise http.Http404

    page = utils.get_page(next)
    if not page:
        raise http.Http404

    set_cookie = False
    if isinstance(request.user, User):
        utils.set_user_display(request, show, models.UserPage, page=page)
    else:
        set_cookie = True
        default, pages = utils.decode_cookie(request)
        pages[page.pk] = show
    response = http.HttpResponseRedirect(next)
    if set_cookie:
        utils.encode_cookie(request, response, default, pages)
    return response



from django.conf.urls.defaults import *

urlpatterns = patterns('pagehelp.views',
    url(r'^hide/$', 'set_default', {'show': False}, name='default_hide'),
    url(r'^show/$', 'set_default', {'show': True}, name='default_show'),
    url(r'^page/show/$', 'set_page', {'show': True}, name='page_show'),
    url(r'^page/hide/$', 'set_page', {'show': False}, name='page_hide'),
)



from django.contrib.sites.models import Site
from django.contrib.sites.managers import CurrentSiteManager
from django.db import models
from django.core.urlresolvers import reverse, NoReverseMatch
import datetime


class SiteSettingManager(models.Manager):
    def get_current(self):
        site = Site.objects.get_current()
        try:
            return site.pagehelp_settings
        except SiteSetting.DoesNotExist:
            return SiteSetting(site=site)


class SiteSetting(models.Model):
    PERMISSIONS = (
        (0, 'Super-users only'),
        (1, 'Staff members'),
        #(2, 'Any registered user'),
    )
    site = models.OneToOneField(Site, related_name='pagehelp_settings')
    display = models.BooleanField(default=True)
    edit_rights = models.PositiveSmallIntegerField('Create/edit rights',
        choices=PERMISSIONS, default=1)
    delete_rights = models.PositiveSmallIntegerField(choices=PERMISSIONS,
        default=0)

    objects = SiteSettingManager()

    def _has_permission(self, user, level):
        if level == 2:
            return user.is_authenticated()
        if level == 1:
            return user.is_staff
        return user.is_superuser

    def can_edit(self, user):
        return self._has_permission(user, self.edit_rights)

    def can_delete(self, user):
        return self._has_permission(user, self.delete_rights)


class Page(models.Model):
    site = models.ForeignKey(Site)
    url = models.CharField(max_length=255, db_index=True)
    text = models.TextField()
    date_created = models.DateTimeField(default=datetime.datetime.now,
                                        editable=False)
    date_edited = models.DateTimeField(editable=False)

    objects = models.Manager()
    on_site = CurrentSiteManager()

    class Meta:
        unique_together = (('site', 'url'),)

    def __unicode__(self):
        return self.url

    def save(self, *args, **kwargs):
        self.date_edited = datetime.datetime.now()
        return super(Page, self).save(*args, **kwargs)
    
    def get_absolute_url(self):
        if '/' in self.url:
            return self.url
        try:
            return reverse(self.url)
        except NoReverseMatch:
            return ''


class UserDefault(models.Model):
    """
    A per-user override of the default display/hide setting for pages on a
    site.

    """
    site = models.OneToOneField(Site, related_name='pagehelp_user_defaults')
    user = models.ForeignKey('auth.User')
    display = models.BooleanField()


class UserPage(models.Model):
    """
    A per-user override of an individual page's display/hide setting.
    
    """
    user = models.ForeignKey('auth.User')
    page = models.ForeignKey(Page, related_name='user_pages')
    display = models.BooleanField()



#!/usr/bin/env python
from distutils.core import setup

setup(name='django-pagehelp',
      version='1.0.1',
      description="A Django application which provides contextual help for "
        "your site's pages.",
      author='Chris Beaven',
      author_email='smileychris@gmail.com',
      url='http://bitbucket.org/smileychris/django-pagehelp/',
      packages=[
          'pagehelp',
      ],
      package_data={'pagehelp': ['templates/*', 'media/*']},
      classifiers=[
          'Development Status :: 4 - Beta',
          'Environment :: Web Environment',
          'Intended Audience :: Developers',
          'License :: OSI Approved :: MIT License',
          'Operating System :: OS Independent',
          'Programming Language :: Python',
          'Framework :: Django',
      ],
)



from django.contrib.auth.models import User
from django.contrib.sites.models import Site
from django.core import urlresolvers
from django.core.cache import cache
from django.utils.http import cookie_date
from pagehelp import models
import datetime
import time

COOKIE_NAME = 'page_display'


def get_default_display(request=None):
    """
    Get the default page display setting.

    If the ``request`` argument is provided, the display setting will first be
    checked specifically for the user.

    If the ``request`` argument was not provided, or the user has not
    overridden the default setting, the default display setting for the current
    site will be calculated (and the result is cached to avoid excessive
    database activity).

    """
    if request:
        if isinstance(request.user, User):
            try:
                return models.UserDefault.objects.get(user=request.user)\
                                                 .display
            except models.UserDefault.DoesNotExist:
                pass
        else:
            return decode_cookie(request)[0]
    site = Site.objects.get_current()
    cache_key = 'pagehelp_display_%s' % site.pk
    display = cache.get(cache_key)
    if display is None:
        site_settings = models.SiteSetting.objects.get_current()
        display = site_settings.display
        cache.set(cache_key, display)
    return display


def set_user_display(request, display, model, **model_kwargs):
    """
    Link a display setting with a user.

    """
    if display is None or not isinstance(request.user, User):
        return

    try:
        obj = model.objects.get(user=request.user, **model_kwargs)
    except model.DoesNotExist:
        obj = None
    if display == get_default_display(request):
        if obj:
            obj.delete()
    else:
        if not obj:
            model.objects.create(user=request.user, display=display,
                                 **model_kwargs)
        elif obj.display != display:
            obj.display = display
            obj.save()


def _decode_cookie_setting(text):
    return {'0': False, '1': True}.get(text)


def _encode_cookie_setting(setting, default=None, always_encode=False):
    """
    Encode the boolean value for display as a string for use in a cookie.

    An empty string is returned if the display matches the default display
    setting (unless ``always_encode`` is ``True``).
 
    """
    mapping = {False: '0', True: '1'}
    if not always_encode:
        if default is None:
            default = get_default_display()
        del mapping[default]
    return mapping.get(setting, '')


def decode_cookie(request):
    """
    Decode the cookie containing all the custom display settings for a user,
    returning a 2-part tuple containing the default setting and a dictionary of
    page settings.

    The keys of the page settings dictionary are page ids and the values are
    a boolean for the related display setting.

    """
    text = request.COOKIES.get(COOKIE_NAME, '')
    default = get_default_display()
    pages = {}
    if text and text.startswith(_encode_cookie_setting(default,
                                                       always_encode=True)):
        bits = text[1:].split(',')
        display = _decode_cookie_setting(bits[0])
        if display is not None:
            default = display
        for page in bits[1:]:
            try:
                page_id, setting = page.split('=')
                page_id = int(page_id)
            except (ValueError, TypeError):
                continue
            pages[page_id] = _decode_cookie_setting(setting)
    return default, pages


def encode_cookie(request, response, default=None, pages=None):
    """
    Encode a cookie with a user's display settings.

    Only settings which differ from the default will be saved. If no settings
    differ, any existing cookie will be deleted.

    """
    site_default = get_default_display()
    text = '%s%s' % (_encode_cookie_setting(site_default, always_encode=True),
                     _encode_cookie_setting(default))
    pages = pages or {}
    for page_id, setting in pages.items():
        setting = _encode_cookie_setting(setting, default=default)
        if not setting:
            continue
        if isinstance(page_id, models.Page):
            page_id = page_id.pk
        text = '%s,%s=%s' % (text, page_id, setting)
    if len(text) > 1:
        delta = datetime.timedelta(days=365)
        max_age = max(0, delta.days * 86400 + delta.seconds)
        expires = cookie_date(time.time() + max_age)
        response.set_cookie(COOKIE_NAME, text, max_age=max_age,
                            expires=expires)
    elif COOKIE_NAME in request.COOKIES:
        response.delete_cookie(COOKIE_NAME)


def get_page(path):
    """
    Get a page object relating to the url ``path`` argument, or ``None`` if no
    matching page exists.

    """
    direct_urls = models.Page.on_site.filter(url=path)
    if direct_urls:
        return direct_urls[0]
    resolver = urlresolvers.get_resolver(None)
    try:
        view = resolver.resolve(path)[0]
    except urlresolvers.Resolver404:
        return
    def find_matching_pattern(resolver):
        for pattern in resolver.url_patterns:
            if isinstance(pattern, urlresolvers.RegexURLResolver):
                match = find_matching_pattern(pattern)
                if match:
                    return match
                continue
            if pattern.callback != view or not pattern.name:
                continue
            named_urls = models.Page.on_site.filter(url=pattern.name)
            if named_urls:
                return named_urls[0]
    return find_matching_pattern(resolver)

def is_visible(request, page):
    """
    Return a boolean as to whether the help should be visible for the given
    user.
 
    """
    if isinstance(request.user, User):
        try:
            return page.user_pages.get(user=request.user).display
        except models.UserPage.DoesNotExist:
            pass
        return get_default_display(request)
    default, pages = decode_cookie(request)
    return pages.get(page.pk, default)
